import torch
from kevin.env_info import version

low_version = version.compare(torch.__version__, "<", "1.10")


def compatible_concat(*args, **kwargs):
    """
        参考 torch.concat()
            在 1.9 版本及其之前的 pytorch 中没有 torch.concat()，
            而只有 torch.cat()，
            两者用法与效果一致， concat 可以看做 cat 的别名
    """

    global low_version
    if low_version:
        return torch.cat(*args, **kwargs)
    else:
        return torch.concat(*args, **kwargs)


if __name__ == '__main__':
    x_ = torch.tensor([[1], [2], [3]])

    print(f"x {x_}, res {compatible_concat((x_, x_), dim=1)}")

    low_version = True

    print(f"x {x_}, res {compatible_concat((x_, x_), dim=1)}")



import pytest
from .compare_version import compare_version
from .parse_version import parse_version_string_to_array


@pytest.mark.parametrize("v_0, operator, v_1, kwargs, result",
                         [([1, 2, 3], ">=", [1, 3], dict(), False),
                          ([1, 2, 3], "<", [1, 3], dict(), True),
                          ([1, 2, 3], "==", [1, 2, 3, 0, 0], dict(), True),
                          # 测试 sep
                          ("1.2.3", "<", [1, 3], dict(sep='.'), True),
                          ("n_7.3", "<", [1, 3], dict(sep='.'), True),
                          ("1_2_3", "==", "1_2_3_0_0", dict(sep='_'), True),
                          # 测试 mode
                          ("1_2_3", "==", "1_2_3_4_5", dict(sep='_', mode='short'), True), ])
def test__compare_version(v_0, operator, v_1, kwargs, result):
    assert compare_version(v_0, operator, v_1, **kwargs) == result


@pytest.mark.parametrize("string, sep, result",
                         [("1,2,3", ',', [1, 2, 3]),
                          ("1_2.3", '.', [0]),
                          ("neg", '_', [0])])
def test__parse_version_string_to_array(string, sep, result):
    assert parse_version_string_to_array(string, sep=sep) == result



import torch
import numpy as np
from kevin.env_info import version

low_version = version.compare(torch.__version__, "<", "1.2")


def compatible_where(*args):
    """
        参考 torch.where()
            在 1.1 版本及其之前的 pytorch 中的 torch.where()，
            只有 torch.where(condition, x, y) 的用法，
            而没有 torch.where(condition) 的用法，
            考虑到兼容性，在pytorch版本过低时，自动使用 numpy.where() 来替代
    """
    global low_version

    if len(args) == 1 and low_version:
        return np.where(args[0].cpu())
    else:
        return torch.where(*args)


if __name__ == '__main__':
    device = torch.device('cuda') if torch.cuda.is_available() else torch.device('cpu')
    x_ = torch.tensor([[1], [2], [3]], device=device)

    print(f"x {x_}, res {compatible_where(x_ > 0)}")

    low_version = True

    print(f"x {x_}, res {compatible_where(x_ > 0)}")



import os
import json


def write_json(content, file_path, sort_keys=False):
    file_path = os.path.abspath(file_path)
    os.makedirs(os.path.dirname(file_path), exist_ok=True)

    with open(file_path, 'w') as f:
        json.dump(content, f, indent=4, ensure_ascii=False, sort_keys=sort_keys)



def parse_version_string_to_array(string, **kwargs):
    """
        将版本的字符串转换为数组的形式
        参数：
            string:                 string of version
            sep：                    分隔符
        工作流程：
            string "1.2.3"  == sep '.' ==> array [1, 2, 3]
            注意：对于分割后包含非整数的 string，默认返回 [0]，例如：
            string "1.2_3"  == sep '_' ==> array [0]
        返回：
            array:                 array of version
    """
    sep = kwargs.get("sep", '.')
    assert isinstance(string, (str,))

    array = string.split(sep, -1)
    for i, v in enumerate(array):
        if v.isdigit():
            array[i] = int(v)
        else:
            # 对于不符合规则的，直接返回 0
            array = [0]
            break
    return array


if __name__ == '__main__':
    print(interpret_version_string("1.2.3"))



import torch
from kevin.env_info import version

low_version = version.compare(torch.__version__, "<", "1.8")


def compatible_tile(x, multiples):
    """
        将 input 的各个维度按照 multiples 中的倍速进行复制（占用内存）
            由于在1.7版本及其之前的 pytorch 没有 torch.tile，
            考虑到兼容性，本函数在pytorch版本过低时自动使用 x.repeat() 来实现 tile 函数

        参数：
            x:                  input tensor
            multiples:          要复制的维度
    """
    assert isinstance(multiples, (list, tuple,))

    global low_version
    if low_version:
        return x.repeat(*multiples)
    else:
        return torch.tile(x, multiples)


if __name__ == '__main__':
    x_ = torch.tensor([[1], [2], [3]])
    multiples_ = (1, 3)

    x_tile = compatible_tile(x_, multiples_)
    print(f"x {x_.shape}, x_tile {x_tile.shape}")
    print(f"x {x_},\n x_tile {x_tile}")

    low_version = True

    x_tile = compatible_tile(x_, multiples_)
    print(f"x {x_.shape}, x_tile {x_tile.shape}")
    print(f"x {x_},\n x_tile {x_tile}")
